# Generated by Django 4.2.19 on 2025-03-17 18:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0005_platformorganization_abbreviation_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='organizationcategory',
            name='data',
            field=models.JSONField(blank=True, default=dict),
        ),
        migrations.AlterField(
            model_name='platformorganization',
            name='data',
            field=models.JSONField(blank=True, default=dict),
        ),
        migrations.AlterField(
            model_name='platformuser',
            name='data',
            field=models.JSONField(blank=True, default=dict),
        ),
    ]
