# Generated by Django 4.2.20 on 2025-03-28 01:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0006_alter_organizationcategory_data_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='organizationcategory',
            options={'verbose_name_plural': 'Organization Categories'},
        ),
    ]
