# Generated by Django 4.2.15 on 2024-09-06 15:53

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("authentication", "0002_organizationmember_platformorganization_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="platformuser",
            name="organizations",
            field=models.ManyToManyField(
                default=None,
                related_name="members",
                through="authentication.OrganizationMember",
                to="authentication.platformorganization",
            ),
        ),
        migrations.AlterField(
            model_name="organizationmember",
            name="platform_organization",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="authentication.platformorganization",
            ),
        ),
        migrations.AlterField(
            model_name="organizationmember",
            name="platform_user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL
            ),
        ),
        migrations.AlterField(
            model_name="platformuser",
            name="email",
            field=models.EmailField(max_length=254, unique=True),
        ),
    ]
